@@ -8,8 +8,9 @@
     for(j in 1:iDataP)
       Data[(i-1)*iDataP+j]=iData[i,j];
   }
- 
-   .C("equSA1",
+  long_mat <- vector("integer",iDataP*iDataP)
+  long_score <- vector("double",3*iDataP*(iDataP-1)/2)
+   result <- .C("equSA1",
       as.numeric(Data),
       as.integer(iMaxNei),
       as.integer(iDataNum),
@@ -18,15 +19,16 @@
       as.numeric(ALPHA2),
       as.integer(GRID),
       as.integer(GRID),
-      as.integer(iteration)
+      as.integer(iteration),
+      as.integer(long_mat),
+      as.numeric(long_score)
       )
-   A <- matrix(scan("sim0.mat"),ncol=iDataP,byrow=TRUE)
-   score <-matrix(scan("sim0.pcor.est.ini"), ncol=3, byrow=TRUE)
+   
+   A <- matrix(result[[10]],ncol=iDataP,byrow=TRUE)
+   score <-matrix(result[[11]], ncol=3, byrow=TRUE)
    result <- list()
    result$Adj = A
    result$score = score
-   unlink("sim0.mat")
-   unlink("sim0.pcor.est.ini")
    return(result)
 }
 
@@ -40,8 +42,8 @@ psical <- function(iData,iMaxNei=as.integer(iDataNum/log(iDataNum)),ALPHA1=0.05,
     for(j in 1:iDataP)
       Data[(i-1)*iDataP+j]=iData[i,j];
   }
-  
-  .C("equSA_sub",
+  long_score <- vector("double",3*iDataP*(iDataP-1)/2)
+  result <- .C("equSA_sub",
      as.numeric(Data),
      as.integer(iMaxNei),
      as.integer(iDataNum),
@@ -49,10 +51,10 @@ psical <- function(iData,iMaxNei=as.integer(iDataNum/log(iDataNum)),ALPHA1=0.05,
      as.numeric(ALPHA1),
      as.integer(GRID),
      as.integer(GRID),
-     as.integer(iteration)
+     as.integer(iteration),
+     as.numeric(long_score)
   )
-  score <-matrix(scan("sim0.pcor.est.ini"), ncol=3, byrow=TRUE)
-  unlink("sim0.pcor.est.ini")
+  score <-matrix(result[[9]], ncol=3, byrow=TRUE)
   return(score)
 }
 
@@ -69,17 +71,17 @@ ContTran <- function(iData,total_iteration=5000,stepsize=0.05)
     for(j in 1:iDataNum)
       Data[(i-1)*iDataNum+j]=iData[i,j];
   }
-  
-  .C("allR",
+  cont_ave <- vector("double",iDataP*iDataNum)
+  result <- .C("allR",
      as.integer(Data),
      as.integer(iDataNum),
      as.integer(iDataP),
      as.integer(total_iteration),
-     as.numeric(stepsize)
+     as.numeric(stepsize),
+     as.numeric(cont_ave)
   )
   
-  continuz <- matrix(scan("continuzed.dat"),ncol=iDataP,byrow=TRUE)
-  unlink("continuzed.dat")
+  continuz <- matrix(result[[6]],ncol=iDataP,byrow=TRUE)
   return(continuz)
 }
 
@@ -88,19 +90,20 @@ ContTran <- function(iData,total_iteration=5000,stepsize=0.05)
    irow=score[,1];
    icol=score[,2];
    idatax=score[,3];
-   length=length(irow);
-   .C("pcorsel1",
+   leng=length(irow);
+   aaa <- vector("double",5*leng)
+   result <- .C("pcorsel1",
       as.integer(irow),
       as.integer(icol),
       as.numeric(idatax),
-      as.integer(length),
+      as.integer(leng),
       as.integer(GRID),
       as.integer(GRID),
-      as.integer(iteration)
+      as.integer(iteration),
+      as.numeric(aaa)
    )
    
-   aaa<-matrix(scan("aaa.fdr"), ncol=5, byrow=TRUE)
-   unlink("aaa.fdr")
+   aaa<-matrix(result[[8]], ncol=5, byrow=TRUE)
    qqq<-ALPHA2
    FFF<-aaa[aaa[,5]<qqq,]
    if(sum(abs(FFF))==0){print("ALPHA2 should be larger!"); qqqscore=max(aaa[,3]);
@@ -111,4 +114,37 @@ ContTran <- function(iData,total_iteration=5000,stepsize=0.05)
      qqqscore<-FFF[1,3]
    }
    return(qqqscore)
+ }
+ 
+ 
+ Mulpval <- function(pvalue,ALPHA2=0.05,GRID=2,iteration=100)
+ {
+   z <- -qnorm(pvalue,lower.tail=TRUE)
+   leng <- length(z)
+   irow=1:leng;
+   icol=irow;
+   idatax=z;
+   aaa <- vector("double",5*leng)
+   result <- .C("pcorsel1",
+                as.integer(irow),
+                as.integer(icol),
+                as.numeric(idatax),
+                as.integer(leng),
+                as.integer(GRID),
+                as.integer(GRID),
+                as.integer(iteration),
+                as.numeric(aaa)
+   )
+   
+   aaa<-matrix(result[[8]], ncol=5, byrow=TRUE)
+   qqq<-ALPHA2
+   FFF<-aaa[aaa[,5]<qqq,]
+   if(sum(abs(FFF))==0){print("ALPHA2 should be larger!"); qqqscore=max(aaa[,3]);
+   }else if(length(FFF)==5){
+     print("ALPHA2 should be larger! Only one pair is selected!");
+     qqqscore <- FFF[3];
+   }else{
+     qqqscore<-FFF[1,3]
+   }
+   return(pnorm(-qqqscore))
  }
\ No newline at end of file
